from BlacklistReport.BlacklistReport import generate_blacklist_report
from BlacklistReport.BlacklistReport import BlacklistReport
from BlacklistReport.BlacklistEntry import BlacklistEntry

import logging


logging.basicConfig(level=logging.WARNING,
                    format='%(name)s - %(levelname)s - %(message)s')
